<?php
namespace ysdmapi\admin;

use ysdmapi\includes\custom_fields\{CustomFieldGroup, CustomFieldsHandler, CustomMetaBox, FieldValidator};
use ysdmapi\includes\PropertyHandler;

/**
 * The admin-specific functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the admin-specific stylesheet and JavaScript.
 *
 * @since 1.0.0
 */
class Admin {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	private $counter;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Undocumented variable
	 *
	 * @var CustomFieldsHandler
	 */
	private $field_handler;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of this plugin.
	 * @param      string    $version    		The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {
		$this->plugin_name = $plugin_name;
		$this->version = $version;
		$this->field_handler = new CustomFieldsHandler();
		$this->define_cutom_meta_boxes();
		$this->counter = 0;

	}

	#region enqueue
	/**
	 * Register the stylesheets for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{
		wp_enqueue_style( $this->plugin_name . '-admin', plugin_dir_url( __FILE__ ) . 'assets/dist/css/ysd-mapi-admin.min.css', array(), $this->version, 'all' );
		wp_enqueue_style('wp-codemirror');

		$this->field_handler->enqueue_styles();

		$current_screen_id = get_current_screen()->id;

		if(strpos($current_screen_id, $this->plugin_name))
		{
			wp_enqueue_style( $this->plugin_name . '-main', plugin_dir_url( __FILE__ ) . 'assets/dist/css/ysd-mapi-main.min.css', array(), $this->version, 'all' );
		}
		
		if($current_screen_id == 'toplevel_page_' . $this->plugin_name)
		{
			wp_enqueue_style( $this->plugin_name . '-settings', plugin_dir_url( __FILE__ ) . 'assets/dist/css/ysd-mapi-settings.min.css', array(), $this->version, 'all' );
		}

		$slug = str_replace('mapi_page_ysd-mapi-', '', $current_screen_id);

		if(file_exists(plugin_dir_path(__FILE__) . 'assets/dist/css/' . $slug . '.css'))
		{
			wp_enqueue_style( $this->plugin_name . '-' . $slug, plugin_dir_url( __FILE__ ) . 'assets/dist/css/' . $slug . '.css', array(), $this->version, 'all' );
		}

	}

	/**
	 * Register the JavaScript for the admin area.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{
		global $pagenow, $typenow;
		$current_screen = get_current_screen()->id;
		if (empty($typenow) && !empty($_GET['post'])) {
			$post = get_post($_GET['post']);
			$typenow = $post->post_type;
		}

		if (is_admin() && $typenow == 'property') {
			if ($pagenow == 'post-new.php' || $pagenow == 'post.php') { 
				// @todo turn into module
				wp_enqueue_script( $this->plugin_name . '-media-view', plugin_dir_url( __FILE__ ) . 'assets/src/js/modules/PropertyMediaView.js', array( 'jquery', 'media-editor' ), $this->version, true );
			}
		}

		if($current_screen == 'toplevel_page_' . $this->plugin_name)
		{
            if(run_user_validation()) {
                wp_enqueue_script( 'jquery-forms', 'https://cdnjs.cloudflare.com/ajax/libs/jquery.form/4.3.0/jquery.form.min.js', array('jquery'), $this->version);
                wp_register_script( $this->plugin_name . '-dashboard', YSD_MAPI_PLUGIN_URL . 'admin/assets/dist/js/ysd-mapi-dashboard.min.js', array( 'jquery', 'jquery-forms' ), $this->version );
                $dashboard_i18n = array(
                    'dateI18n' => __( 'Date', YSD_MAPI_TEXT_DOMAIN ),
                    'statusI18n' => __( 'Status', YSD_MAPI_TEXT_DOMAIN ),
                    'redoI18n' => __( 'Redo', YSD_MAPI_TEXT_DOMAIN ),
                    'timeI18n' => __( 'Time', YSD_MAPI_TEXT_DOMAIN ),
                    'mediaI18n' => __( 'Media', YSD_MAPI_TEXT_DOMAIN ),
                    'failedI18n' => __( 'Failed', YSD_MAPI_TEXT_DOMAIN ),
                    'updatedI18n' => __( 'Updated', YSD_MAPI_TEXT_DOMAIN ),
                    'addedI18n' => __( 'Added', YSD_MAPI_TEXT_DOMAIN ),
                    'errorI18n' => __( 'An error has occurred', YSD_MAPI_TEXT_DOMAIN ),
                    'noneI18n' => __( 'No schedule', YSD_MAPI_TEXT_DOMAIN ),
                    'updateInfoStat' => [
                        '-1' => __( 'Cancelled', YSD_MAPI_TEXT_DOMAIN ),
                        '0' => __( 'Connecting to API', YSD_MAPI_TEXT_DOMAIN ),
                        '1' => __( 'Processing properties', YSD_MAPI_TEXT_DOMAIN ),
                        '2' => __( 'Deleting old properties', YSD_MAPI_TEXT_DOMAIN ),
                        '3' => __( 'Finished', YSD_MAPI_TEXT_DOMAIN ),
                    ],
                    'siteURL' => site_url(),
                );
                wp_localize_script( 'ysd-mapi-dashboard', 'dashboardI18n', $dashboard_i18n );
                wp_enqueue_script( 'ysd-mapi-dashboard', array(), '', true );
            }
		}

		$cm_settings = array(
            'ajaxurl' => admin_url( 'admin-ajax.php' ),
            'codeEditor' => wp_enqueue_code_editor(array('type' => 'text/css')),
            'overlayI18n' => __( 'Click to edit!', YSD_MAPI_TEXT_DOMAIN ),
            'resetI18n' => __( 'Are you sure you want to reset the filters?', YSD_MAPI_TEXT_DOMAIN ),
            'updateI18n' => __( 'Are you sure you want to update now?', YSD_MAPI_TEXT_DOMAIN ),
            'resetAllI18n' => __( 'Are you sure you want to reset all to defaults?', YSD_MAPI_TEXT_DOMAIN ),
        );
		wp_localize_script('jquery', 'cmSettings', $cm_settings);

		wp_enqueue_script( $this->plugin_name . '-admin', plugin_dir_url( __FILE__ ) . 'assets/dist/js/ysd-mapi-admin.min.js', array( 'jquery' ), $this->version );

	}

	/**
	 * Method to be hooked to script_loader_tag filter
	 * 
	 * Changes the way certain scripts are loaded
	 *
	 */
	public function script_load_filter($tag, $handle, $src) {
		$module_scripts = [
			$this->plugin_name . '-admin',
			$this->plugin_name . '-dashboard',
		];

		/* 
		 * if plugin admin script, change the script tag by
		 * adding type="module". 
		*/
		if (in_array($handle, $module_scripts))
		{
			$tag = '<script type="module" src="' . esc_url( $src ) . '"></script>';
		}

		return $tag;

	}
	
	#endregion

	#region admin navigation
	public function add_plugin_menu()
	{
		add_menu_page('MAPI', 'MAPI', 'administrator', $this->plugin_name, array($this, 'display_plugin_admin_page'), 'dashicons-admin-home', 31);
		add_submenu_page($this->plugin_name, __('Dashboard', YSD_MAPI_TEXT_DOMAIN), __('Dashboard', YSD_MAPI_TEXT_DOMAIN), 'administrator', $this->plugin_name, array($this, 'display_plugin_admin_page'), 0);

	}

	/**
	 * Displays the admin page that corresponds with the page slug from the partials folder
	 *
	 * @return void
	 */
	public function display_plugin_admin_page()
	{
		global $plugin_page;

		require_once plugin_dir_path(__FILE__) . 'partials/' . $plugin_page . '.php';
		
	}
	#endregion

	#region custom meta boxes
	public function define_cutom_meta_boxes()
	{
		include_once YSD_MAPI_PLUGIN_PATH . 'admin/meta-fields/meta-employee.php';
		include_once YSD_MAPI_PLUGIN_PATH . 'admin/meta-fields/meta-office.php';
		$property_field_groups =  require_once plugin_dir_path(__FILE__) . 'field-declerations/property-fields.php';
		$property_field_groups = $this->field_handler->create_field_groups_by_array($property_field_groups);

		$property_meta_box = new CustomMetaBox('property_data', 'Property', 'property', 'tabbed_box', 'normal');
		$property_meta_box->add_custom_field_groups($property_field_groups);

		$this->field_handler->add_metabox($property_meta_box);

	}

	public function register_custom_meta_boxes()
	{
		$this->field_handler->register_meta_boxes();
	}

	public function save_postdata($post_id)
	{
		$this->field_handler->save_meta_boxes($post_id);
	}

	public function validate_field($request)
	{
		$field_id = $request['id'];
		$value = $request['value'];
		$results['valid'] = true;

		$field = $this->field_handler->get_field($field_id);

		if(isset($field))
		{
			$results = FieldValidator::validate_field($field, $value);
		}

		return rest_ensure_response($results);

	}
	#endregion

	#region custom settings
	public function add_custom_settings()
	{
		$setting_groups = require_once plugin_dir_path(__FILE__) . 'field-declerations/admin-settings.php';
		$setting_groups = $this->field_handler->create_field_groups_by_array($setting_groups);

		foreach($setting_groups as $setting_group)
		{
			$setting_group->add_pages($this->plugin_name);
			$this->field_handler->add_field_group($setting_group);

			$this->register_custom_setting_group($setting_group);
		}

	}

	private function register_custom_setting_group(CustomFieldGroup $section)
	{
		foreach($section->get_fields() as $setting)
		{
			// Quickfix for prefix @todo add prefix functionality to fieldshandler
			$id = $this->plugin_name . '_' . $setting->id;
			$setting->id = $id;

			$default_value = $setting->value ?? false;

			/* check if setting has [] part for array fields */
			$matches = [];
			$bracket_part = preg_match('/\[(.+)\]/', $id, $matches);

			if($bracket_part)
			{
				// Remove [] array part for registering the setting
				$id = preg_replace('/\[.+\]/', '', $id);

				$value = get_option($id, $default_value);

				/* Get the field value matching the key from the [] part */
				$key = $matches[1];

				if(is_array($value) && isset($value[$key]))
				{
					$value = $value[$key];
				}
			}
			else
			{
				$value = get_option($id, $default_value);
			}

			$setting->value = $value;

			// Register the field as a wordpress setting
			register_setting($this->plugin_name, $id);
		}

	}
	#endregion

	#region Rest endpoints

	#region log_files
	public function get_logs($request)
	{
		$name = $request->get_param('name') ?: '*';
		$year = $request->get_param('year') ?: '[0-9][0-9][0-9][0-9]';
		$month = $request->get_param('month') ?: '[0-9][0-9]';
		$day = $request->get_param('day') ?: '[0-9][0-9]';

		$upload_dir_array = wp_get_upload_dir();
		$results = array();

		$log_files = glob($upload_dir_array['basedir'] . '/' . $this->plugin_name . "/logs/$year/$month/$day/" . $name . '_*.log');

		if(empty($log_files))
		{
			return rest_ensure_response('No resources found');
		}

		if($request->get_param('last'))
		{
			$log_files = [end($log_files)];
		}

		foreach($log_files as $file)
		{
			$matches = array();
			preg_match('/[0-9]+/', basename($file), $matches);

			$results[$matches[0]] = json_decode(file_get_contents($file), true);
		}

		krsort($results, SORT_NUMERIC);

		return rest_ensure_response($results);

	}

	public function scanLogs()
	{
		$dir = wp_get_upload_dir()['basedir'] . '/' . $this->plugin_name . '/logs';

		return $this->dirToArray($dir);

	}

	function dirToArray($dir) {
		$result = array();
		$cdir = scandir($dir);

		unset($cdir[array_search('.', $cdir, true)]);
		unset($cdir[array_search('..', $cdir, true)]);
		unset($cdir[array_search('last', $cdir, true)]);
		$cdir = array_values($cdir);

		$result = [];

		foreach ($cdir as $value) 
		{
			if (is_dir($dir . DIRECTORY_SEPARATOR . $value))
			{
				$result[$value] = $this->dirToArray($dir . DIRECTORY_SEPARATOR . $value);
			}
		}

		return $result;
	}
	#endregion

	#region cron schedules
	public function get_next_cron($request)
	{
		$hook = $request->get_param('hook');

		return rest_ensure_response(wp_next_scheduled($hook));
	}
	#endregion
	
	#region update job
	public function resetUpdateJob($request)
	{
		$timestamp = $request->get_param('timestamp');

		if($timestamp)
		{
			$date = date('Y/m/d', $timestamp);

			$propertyHandler = new PropertyHandler();

			$propertyHandler->plugin_name = $this->plugin_name;

			$upload_dir_array = wp_get_upload_dir();

			$file = $upload_dir_array['basedir'] . '/' . $this->plugin_name . '/logs/' . $date . '/crm_data' . '_' . $timestamp . '.log';

			$data = json_decode(file_get_contents($file), true);

			$propertyHandler->process_properties($data);

			return 'yes!';
		}

	}
	#endregion

	#region employees
	public function mapBusiness($request) {

        // Employees
		$accountmanagers = $request->get_param('accountmanagers');
		foreach($accountmanagers as $key => $value) {
            update_post_meta($key, 'accountmanager', $value);
		}
        
        // Offices
		$agencies = $request->get_param('agencies');
		foreach($agencies as $key => $value) {
            update_post_meta($key, 'agency_id', $value);
		}

		$location = wp_get_referer();
        wp_safe_redirect($location);
		exit();
	}
	#endregion

	#endregion

	#region media library
	/**
	 * filters posts/attachments based on show_in_library meta value
	 *
	 * @param [type] $query
	 * @return void
	 */
	public function filter_media($query)
	{
		if(!is_admin())
		{
			return;
		}
		
		$query->set( 'meta_query', [
			'relation' => 'OR',
			[
				'key' => 'show_in_library',
				'value' => false,
				'compare' => '!=',
			],
			[
				'key' => 'show_in_library',
				'compare' => 'NOT EXISTS',
			],
		]);
	
		return $query;
	}
	
	/**
	 * filters posts/attachments based on show_in_library meta value on ajax calls
	 *
	 * @param [type] $args
	 * @return void
	 */
	public function filter_ajax_media($args)
	{
		if ( ! is_admin() ) {
			return;
		}

		if(!isset($args['post_parent']) || get_post($args['post_parent'])->post_type !== "property")
		{
			$args['meta_query'] = [
				'relation' => 'OR',
				[
					'key' => 'show_in_library',
					'value' => false,
					'compare' => '!=',
				],
				[
					'key' => 'show_in_library',
					'compare' => 'NOT EXISTS',
				],
			];
		}
	
		return $args;
	}
	#endregion

}
