<?php
namespace ysdmapi\Front;

/**
 * The public-facing functionality of the plugin.
 *
 * Defines the plugin name, version, and two examples hooks for how to
 * enqueue the public-facing stylesheet and JavaScript.
 *
 * @since 1.0.0
 */
class Front {

	/**
	 * The ID of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $plugin_name    The ID of this plugin.
	 */
	private $plugin_name;

	/**
	 * The version of this plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 * @var      string    $version    The current version of this plugin.
	 */
	private $version;

	/**
	 * Initialize the class and set its properties.
	 *
	 * @since    1.0.0
	 * @param      string    $plugin_name       The name of the plugin.
	 * @param      string    $version    The version of this plugin.
	 */
	public function __construct( $plugin_name, $version ) {

		$this->plugin_name = $plugin_name;
		$this->version = $version;

        // Load Public
        $this->load_public();
	}

	/**
	 * Register the stylesheets for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_styles()
	{
		wp_enqueue_style( $this->plugin_name . '-property-archive', plugin_dir_url( __FILE__ ) . 'assets/dist/css/property-archive.min.css', array(), $this->version, 'all' );
		wp_enqueue_style( $this->plugin_name . '-property-single', plugin_dir_url( __FILE__ ) . 'assets/dist/css/property-single.min.css', array(), $this->version, 'all' );
	}

	/**
	 * Register the JavaScript for the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function enqueue_scripts()
	{
		wp_enqueue_script( $this->plugin_name . '-property-archive', plugin_dir_url( __FILE__ ) . 'assets/dist/js/property-archive.min.js', array('jquery'), $this->version, false );
		wp_enqueue_script( $this->plugin_name . '-property-single', plugin_dir_url( __FILE__ ) . 'assets/dist/js/property-single.min.js', array('jquery'), $this->version, false );// Load script on demand
        
        // Load script on demand
        $settings = array(
            'defaultPropertyLayout' => get_option( 'ysd-mapi_archive_properties_layout', 'grid' ),
        );
        wp_localize_script( 'ysd-mapi-property-archive', 'archiveSettings', $settings );
        // Enqueued script with localized data.
        wp_enqueue_script( 'ysd-mapi-property-archive', array(), '', true );

        // Load script on demand
        if ( is_singular( 'property' ) ) {

            wp_localize_script( 'ysd-mapi-property-single', 'data', get_property_js_data() );
            // Enqueued script with localized data.
            wp_enqueue_script( 'ysd-mapi-property-single', array(), '', true );
        }
	}

	/**
	 * Function to be hooked to the template_include filter.
	 * 
	 * Makes sure certain template-files are loaded from the plugin folder,
	 * when they don't exist inside the theme folder.
	 *
	 * @since 1.0.0
	 * 
	 * @param mixed $template
	 * @return mixed
	 */
	public function real_estate_template( $template )
	{
		if(get_post_type() == 'property')
		{
			$type = is_singular('property') ? 'single' : 'archive';

			$template_in_theme = locate_template($type . '-property.php', false);
			
			if ( empty($template_in_theme) ) {
				$template = plugin_dir_path(__FILE__) . 'partials/templates/' . $type . '-property.php';
			}
		}

		// Offer: Buy
        $get_buy_template = get_option( 'ysd-mapi_buy_template' ) ?? 1;
        $get_current_buy_template = apply_filters( 'wpml_object_id', $get_buy_template, 'attachment', FALSE );
        $buy_template_id = ! empty( $get_current_buy_template ) ? $get_current_buy_template : $get_buy_template;

        // Check Offer!
        if ( ! empty( $buy_template_id ) && is_page( $buy_template_id ) ) {

            $taxonomy = 'offer';

            $template_in_theme = locate_template( 'taxonomy-'. $taxonomy . '.php', false );
            
            if ( empty( $template_in_theme ) ) {
                $template = plugin_dir_path(__FILE__) . 'partials/templates/taxonomy-' . $taxonomy . '.php';
            }

        }

        // Offer: Rent
        $get_rent_template = get_option( 'ysd-mapi_rent_template' ) ?? 1;
        $get_current_rent_template = apply_filters( 'wpml_object_id', $get_rent_template, 'attachment', FALSE );
        $rent_template_id = ! empty( $get_current_rent_template ) ? $get_current_rent_template : $get_rent_template;

        // Check Offer!
        if ( ! empty( $rent_template_id ) && is_page( $rent_template_id ) ) {

            $taxonomy = 'offer';

            $template_in_theme = locate_template( 'taxonomy-'. $taxonomy . '.php', false );
            
            if ( empty( $template_in_theme ) ) {
                $template = plugin_dir_path(__FILE__) . 'partials/templates/taxonomy-' . $taxonomy . '.php';
            }

        }

		return $template;

    }
    
	/**
	 * Register the PHP and HTML for the public-facing side of the site.
	 *
	 * @since 1.0.0
	 */
    public function load_public() {

		/**
		 * The class responsible for defining all actions that occur on the front-end.
		 */
        require_once YSD_MAPI_PLUGIN_PATH . '/front/partials/public.php';
    }
    
	/**
	 * Enqueue CSS for unrelated Yourstyle Plus themes.
	 *
	 * @since 1.0.0
	 */
    public function load_non_ysp_styles() {

        if ( ! is_ysp_theme() ) {
            wp_enqueue_style( $this->plugin_name . '-default-frontend', plugin_dir_url( __FILE__ ) . 'assets/dist/css/default-frontend.min.css', array(), $this->version, 'all' );
        }
    }

}
