<?php

namespace ysdmapi\includes;

use WP_Error;

class ApiHandler 
{
    protected string $url;

    public function __construct(string $url)
    {
        $this->url = $url;
    }

    /**
     * Executes an HTTP request to the URL and adds the API key as parameter
     *
     * @param array $variables
     * The get parameters to pass with the request
     * 
     * @param array $headers
     * The headers used by the request
     * 
     * @param array|string $retrieve
     * An array of keys to retrieve from the response
     * 
     * @return mixed
     * the body of the api response or a wordpress error on failure
     */
    public function fetch_data(array $variables = array(), array $headers = array(), $retrieve = '')
    {
        $url = $this->append_variables_to_url($variables);

        $args['headers'] = $headers;

        $test_response = wp_remote_head($url, $args);
        $response_status = wp_remote_retrieve_response_code($test_response);

        if($response_status < 200 || $response_status >= 300 || empty($response_status))
        {
            return new WP_Error('failed_request',
             sprintf(
                __('The request failed with status code: %d', YSD_MAPI_TEXT_DOMAIN), $response_status),
                ['status_code' => $response_status,
                'timestamp' => time()]
            );
        }

        $response = wp_remote_get($url, $args);
        $decoded_response = json_decode(wp_remote_retrieve_body($response), true);

        //$decoded_response = $this->get_items_from_response($retrieve, $decoded_response);

        return $decoded_response;
    
    }

    /**
     * Appends given variables to the url string as GET parameters
     *
     * @param array $variables
     * @return string
     */
    private function append_variables_to_url(array $variables) : string
    {
        $url = $this->url;

        if(!strpos($url, '?') && !empty($variables))
        {
            $url = $url . '?';
        }

        $i = 0;

        foreach($variables as $key => $value)
        {
            if($i != 0)
            {
                $url .= '&';
            }

            $url .= $key . '=' . $value;

            $i++;
        }

        return $url;

    }

    /**
     * Retrieves the specified items from the response
     *
     * @param array|string $retrieve
     * The items to retrieve from the response
     * 
     * @param array $decoded_response
     * The response as an associative array
     * 
     * @return mixed
     * The items from the response
     */
    private function get_items_from_response($retrieve, $decoded_response)
    {
        if(!empty($retrieve) && is_array($decoded_response) && !empty($decoded_response))
        {
            if(is_array($retrieve))
            {
                // Make sure all passed values in $retrieve are considered keys
                array_walk_recursive($retrieve, function(&$item, $key) {
                    if(is_array($item))
                    {
                        foreach($item as $value)
                        {
                            // If child is not an array make it the key
                            if(!is_array($value))
                            {
                                $item = array_flip($item);
                            }
                        }
                    }
                });

                echo "<pre>";
                die(print_r($retrieve));

                //Return only the keys that exist in $retrieve
                $decoded_response = $this->array_intersect_key_recursive($decoded_response, $retrieve);
            }
            else
            {
                $decoded_response = $decoded_response[$retrieve] ?? array();
            }
        }

        return $decoded_response;
    }

    /** 
    * Recursively computes the intersection of arrays using keys for comparison.
    * 
    * @param   array $array1 The array with master keys to check.
    * @param   array $array2 An array to compare keys against.
    * @return  array associative array containing all the entries of array1 which have keys that are present in array2.
    **/
    private function array_intersect_key_recursive(array $array1, array $array2) {
        $array1 = array_intersect_key($array1, $array2);

        foreach ($array1 as $key => &$value) {
            if (is_array($value) && is_array($array2[$key])) {
                $value = $this->array_intersect_key_recursive($value, $array2[$key]);
            }
        }

        return $array1;
    }

    public function setUrl(string $url) : void
    {
        $this->url = $url;
    }

}