<?php
namespace ysdmapi\includes;

/**
 * Automatically requires php classes (that use the given namespace prefix and
 * are in the given base directory) when they are instantiated.
 * 
 * @since 1.0.0
 * 
 * @package ysdmapi
 * @subpackage ysdmapi\include
 */
class Autoloader {
    #region properties

    /**
     * @var string The base-namespace/prefix of the classes that should be autoloaded
     */
    protected string $namespace_prefix;
    
    /**
     * @var string The base/root directory of the classes that should be autoloaded
     */
    protected string $base_dir_path;

    #endregion

    /**
     * @param string $namespace_prefix
     * The base-namespace/prefix of the classes that should be autoloaded
     * 
     * @param string $base_dir_path
     * The base/root directory of the classes that should be autoloaded
     */
    function __construct(string $namespace_prefix, string $base_dir_path) {
        $this->namespace_prefix = $namespace_prefix;
        $this->base_dir_path = $base_dir_path;
    }
    
    #region methods

    /**
     * Registers the AutoLoader with the spl_autoload_register method, so it
     * runs everytime a class is instantiated.
     *
     * @return void
     */
    public function activate()
    {
        spl_autoload_register(array($this, 'class_load_handler'));
    }

    #region private methods

    /**
     * Handless the auto loading of classes within the given namespace,
     * by automatically requiring the classes when they are instanciated.
     * 
     * Gets called when a class is instantiated.
     * 
     * @param string $class 
     * A string with the full namespace + classname of the class that should be required
     * 
     * @return void
     */
    private function class_load_handler(string $class) {
        // project base namespace
        $prefix = $this->namespace_prefix;

        // base directory for the project
        $base_dir = $this->base_dir_path;

        // does the class use the base namespace?
        $len = strlen($prefix);
        if (strncmp($prefix, $class, $len) !== 0) {
            // no, skip this autoloader and move to the next registered autoloader
            // in this application (if there is one)
            return;
        }

        // get the relative class name (class name without the namespace prefix)
        $relative_class = substr($class, $len);

        // replace the namespace prefix with the base directory, replace namespace
        // separators with directory separators in the relative class name, append
        // with .php
        $file = $base_dir . str_replace('\\', '/', $relative_class) . '.php';

        // if the file exists, require it
        if (file_exists($file)) {
            require $file;
        }

    }

    #endregion

    #endregion
    
    #region getters/setter

    /**
     * Get the base-namespace/prefix of the classes that should be autoloaded
     *
     * @return  string
     */ 
    public function get_namespace_prefix()
    {
        return $this->namespace_prefix;
    }

    /**
     * Set the base-namespace/prefix of the classes that should be autoloaded
     *
     * @param  string  $namespace_prefix  The base-namespace/prefix of the classes that should be autoloaded
     *
     * @return  self
     */ 
    public function set_namespace_prefix(string $namespace_prefix)
    {
        $this->namespace_prefix = $namespace_prefix;

        return $this;
    }

    /**
     * Get the base/root directory of the classes that should be autoloaded
     *
     * @return  string
     */ 
    public function get_base_dir_path()
    {
        return $this->base_dir_path;
    }

    /**
     * Set the base/root directory of the classes that should be autoloaded
     *
     * @param  string  $base_dir_path  The base/root directory of the classes that should be autoloaded
     *
     * @return  self
     */ 
    public function set_base_dir_path(string $base_dir_path)
    {
        $this->base_dir_path = $base_dir_path;

        return $this;
    }

    #endregion
}
