<?php
namespace ysdmapi\includes;

use ysdmapi\admin\Admin;
use ysdmapi\front\Front;

/**
 * The file that defines the core plugin class
 *
 * A class definition that includes attributes and functions used across both the
 * public-facing side of the site and the admin area.
 *
 * @since      1.0.0
 *
 * @package    ysdmapi
 * @subpackage ysdmapi/includes
 */
class Mapi
{

	#region class properties
	/**
	 * The loader that's responsible for maintaining and registering all hooks that power
	 * the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      Loader    $loader    Maintains and registers all hooks for the plugin.
	 */
	protected $loader;

	/**
	 * The unique identifier of this plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $plugin_name    The string used to uniquely identify this plugin.
	 */
	protected $plugin_name;

	/**
	 * The current version of the plugin.
	 *
	 * @since    1.0.0
	 * @access   protected
	 * @var      string    $version    The current version of the plugin.
	 */
	protected $version;

	/**
	 * The admin class used by the plugin
	 *
	 * @var ysdmapi\admin\Admin
	 */
	protected $admin;
	#endregion

	#region constructor
	/**
	 * Define the core functionality of the plugin.
	 *
	 * Set the plugin name and the plugin version that can be used throughout the plugin.
	 * Load the dependencies, define the locale, and set the hooks for the admin area and
	 * the public-facing side of the site.
	 *
	 * @since    1.0.0
	 */
	public function __construct()
	{
		if ( defined( 'YSD_MAPI_VERSION' ) ) {
			$this->version = YSD_MAPI_VERSION;
		} else {
			$this->version = '1.0.0';
		}
		$this->plugin_name = 'ysd-mapi';

		$this->load_dependencies();
		$this->set_locale();
		$this->define_admin_hooks();
		$this->define_public_hooks();
		$this->define_hooks();

	}

	/**
	 * Load the required dependencies for this plugin.
	 *
	 * Create an instance of the loader which will be used to register the hooks
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function load_dependencies()
	{
		$this->admin = new Admin( $this->get_plugin_name(), $this->get_version() );
		$this->loader = new Loader();
	}
	#endregion

	#region run
	/**
	 * Run the loader to execute all of the hooks with WordPress.
	 *
	 * @since    1.0.0
	 */
	public function run()
	{
		$updateChecker = new UpdateChecker();
		$updateChecker->start();
        
        $this->loader->run();
	}
	#endregion

	#region internationalization
	/**
	 * Define the locale for this plugin for internationalization.
	 *
	 * Uses the i18n class in order to set the domain and to register the hook
	 * with WordPress.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function set_locale()
	{
		$plugin_i18n = new I18n();

		$this->loader->add_action( 'plugins_loaded', $plugin_i18n, 'load_plugin_textdomain' );

	}
	#endregion

	#region cron jobs
	public function schedule_cron_jobs()
	{
		$sync = get_option($this->plugin_name . '_sync_data');
		$schedule = get_option($this->plugin_name . '_sync_schedule');
		$recurrence = 'daily';
		$time = time();

		switch($schedule['type'])
		{
			case 'daily':
				$recurrence = 'daily';
				$time = strtotime($schedule['tod'] . ' Europe/Amsterdam');
				break;
			case 'hourly':
				$recurrence = 'mapi_custom_hours';
				break;
		}

		/* unschedule hook before rescheduling */
		if(wp_next_scheduled('ysd_mapi_api_cron_hook')) {
			$timestamp = wp_next_scheduled( 'ysd_mapi_api_cron_hook' );
			wp_unschedule_event( $timestamp, 'ysd_mapi_api_cron_hook' );
		}

		if ( ! wp_next_scheduled( 'ysd_mapi_api_cron_hook' ) && $sync) {
			wp_schedule_event( $time, $recurrence, 'ysd_mapi_api_cron_hook' );
		}

	}

	/**
	 * Add custom cron schedules to wordpress
	 *
	 * @param array $schedules
	 * @return void
	 */
	public function cron_add_schedules($schedules)
	{
		$custom_schedules = get_option($this->plugin_name . '_sync_schedule');

		if($custom_schedules)
		{
			if(is_numeric($custom_schedules['hourly_interval']))
			{
				$schedules['mapi_custom_hours'] = [
					'interval' => intval($custom_schedules['hourly_interval']) * 3600,
					'display' => __( 'Mapi custom hours' , YSD_MAPI_TEXT_DOMAIN),
				];
			}
		}

		return $schedules;
	}
	#endregion

	#region hooks
	/**
	 * Register all of the hooks related to the admin area functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_admin_hooks()
	{
        $this->loader->add_action( 'admin_init', $this->admin, 'add_custom_settings' );
        $this->loader->add_action( 'admin_enqueue_scripts', $this->admin, 'enqueue_styles' );
        $this->loader->add_action( 'admin_enqueue_scripts', $this->admin, 'enqueue_scripts' );
		$this->loader->add_action( 'admin_menu', $this->admin, 'add_plugin_menu');

        if(run_user_validation()) {
            $this->loader->add_action( 'add_meta_boxes', $this->admin, 'register_custom_meta_boxes');
            $this->loader->add_action( 'save_post', $this->admin, 'save_postdata' );
            
            $this->loader->add_filter( 'ajax_query_attachments_args', $this->admin, 'filter_ajax_media' );
            $this->loader->add_filter( 'script_loader_tag', $this->admin, 'script_load_filter', 10, 3);
        }

	}

	/**
	 * Register all of the hooks related to the public-facing functionality
	 * of the plugin.
	 *
	 * @since    1.0.0
	 * @access   private
	 */
	private function define_public_hooks()
	{
        if(run_user_validation()) {
            $plugin_public = new Front( $this->get_plugin_name(), $this->get_version() );

            $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_styles' );
            $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'enqueue_scripts' );
            $this->loader->add_action( 'wp_enqueue_scripts', $plugin_public, 'load_non_ysp_styles' );

            $this->loader->add_filter( 'template_include', $plugin_public, 'real_estate_template' );
        }

	}

	/**
	 * Register all of the hooks related to functionality that are shared between the admin area and 
	 * the public-facing parts of the plugin.
	 *
	 * @return void
	 */
	private function define_hooks()
	{

        if(run_user_validation()) {
            $property_handler = new PropertyHandler();
            $property_handler->plugin_name = $this->plugin_name;

            $this->loader->add_action( 'init', $this, 'register_custom_post_types' );
            $this->loader->add_action( 'init', $this, 'register_custom_taxonomies', 0 );
            $this->loader->add_action( 'rest_api_init', $this, 'register_api_routes' );
            $this->loader->add_action( 'ysd_mapi_api_cron_hook', $property_handler, 'sync_api_data' );
            $this->loader->add_action( 'before_delete_post', $property_handler, 'delete_property_media' );
            $this->loader->add_action( 'load-upload.php', $this, 'load_media_library' );
            $this->loader->add_action( 'delete_attachment', $this, 'deleteAttachmentTranslations' );

            if ( get_option($this->plugin_name . '_sync_data') ) {
                $this->loader->add_action( 'update_option_' . $this->plugin_name . '_mapi_token', $property_handler, 'sync_api_data' );
                $this->loader->add_action( 'wp_ajax_ajaxrunmapicron', $property_handler, 'sync_api_data' );
                // $this->loader->add_action( 'wp_ajax_ajaxrunmapicron', $this, 'immediate_sync_api_data' );
            }

            $this->loader->add_action( 'update_option_' . $this->plugin_name . '_show_in_library', $property_handler, 'update_show_in_library' );
            $this->loader->add_action( 'update_option_' . $this->plugin_name . '_sync_schedule', $this, 'schedule_cron_jobs' );
            $this->loader->add_action( 'update_option_' . $this->plugin_name . '_sync_data', $this, 'schedule_cron_jobs' );

            $this->loader->add_filter( 'excerpt_more', $this, 'custom_excerpt_more' );
            $this->loader->add_filter( 'excerpt_length', $this, 'custom_excerpt_length' );
            $this->loader->add_filter( 'cron_schedules', $this, 'cron_add_schedules' );
            $this->loader->add_filter( 'intermediate_image_sizes_advanced', $this, 'filterImageSizes', 10, 3 );
        }

	}
	#endregion

	#region REST API
	
	#region routes
	public function register_api_routes()
	{
		// @todo: Add authentication

		$version = '1';
		$namespace = $this->plugin_name . '/v' . $version;

		/* Input validation */
		register_rest_route( $namespace, '/input-validation/(?P<id>[a-z-_0-9]+)', [
		  'methods' => 'GET',
		  'callback' => [$this->admin, 'validate_field'],
		]);

		/* Log routes */
		register_rest_route( $namespace, '/logs', [
			'methods' => 'GET',
			'callback' => [$this->admin, 'scanLogs'],
			'args' => $this->get_log_args(),
		]);
		register_rest_route( $namespace, '/logs/(?P<name>[a-z_]+)', [
			'methods' => 'GET',
			'callback' => [$this->admin, 'get_logs'],
			'args' => $this->get_log_args(),
		]);
		register_rest_route( $namespace, '/logs/(?P<year>\d{4})/(?P<month>\d{2})/(?P<day>\d{2})', [
			'methods' => 'GET',
			'callback' => [$this->admin, 'get_logs'],
			'args' => $this->get_log_args(),
		]);
		register_rest_route( $namespace, '/logs/(?P<year>\d{4})/(?P<month>\d{2})', [
			'methods' => 'GET',
			'callback' => [$this->admin, 'get_logs'],
			'args' => $this->get_log_args(),
		]);
		register_rest_route( $namespace, '/logs/(?P<year>\d{4})', [
			'methods' => 'GET',
			'callback' => [$this->admin, 'get_logs'],
			'args' => $this->get_log_args(),
		]);

		/* Cron job routes */
		register_rest_route( $namespace, '/cron-jobs/next/(?P<hook>[a-z_]+)', [
			'methods' => 'GET',
			'callback' => [$this->admin, 'get_next_cron'],
		]);

		/* Update job routes */
		register_rest_route( $namespace, '/update-job/reset/(?P<timestamp>\d+)', [
			'methods' => 'POST',
			'callback' => [$this->admin, 'resetUpdateJob'],
		]);

		/* Map Business */
		register_rest_route( $namespace, '/business', [
            'methods' => 'POST',
			'callback' => [$this->admin, 'mapBusiness'],
		]);
	}
	#endregion

	#region arguments
	public function get_log_args()
	{
		$args = array();

		$args['last'] = [
			'description' => esc_html__( 'The filter parameter is used to filter the collection of logs', YSD_MAPI_TEXT_DOMAIN ),
			'type' => 'boolean',
		];

		return $args;
	}
	#endregion

	#endregion

	#region custom post types
	/**
	 * Registration of plug-ins custom post types
	 * 
	 * @todo add naming collition prevention
	 *
	 * @return void
	 */
	public function register_custom_post_types()
	{
        $archive_rewrite_slug = get_option( 'ysd-mapi_archive_rewrite_slug', 'property' );
        
        $reva = get_option( 'ysd-mapi_api_handler_url', 0 );
		$taxonomies = $reva == 1 ? array('agency') : array();

		$taxonomies = apply_filters('ysd_mapi_property_taxonomies', $taxonomies);
		
		/* Define the custom post types here */
		$custom_post_types['property'] = array(
			'labels' => array(
				'name' => __('Properties', YSD_MAPI_TEXT_DOMAIN),
				'singular_name' => __('Property', YSD_MAPI_TEXT_DOMAIN),
				'add_new' => __('Add Property', YSD_MAPI_TEXT_DOMAIN),
				'add_new_item' => __('Add New Property', YSD_MAPI_TEXT_DOMAIN),
				'edit_item' => __('Edit Property', YSD_MAPI_TEXT_DOMAIN),
				'new_item' => __('New Property', YSD_MAPI_TEXT_DOMAIN),
				'view_item' => __('View Property', YSD_MAPI_TEXT_DOMAIN),
				'search_items' => __('Search Properties', YSD_MAPI_TEXT_DOMAIN),
				'not_found' => __('No Properties found', YSD_MAPI_TEXT_DOMAIN),
				'not_found_in_trash' => __('No Properties found in trash', YSD_MAPI_TEXT_DOMAIN),
				'parent_item_colon' => __('Property:', YSD_MAPI_TEXT_DOMAIN),
			),
			'supports' => array(
				'title',
				'editor',
				'thumbnail',
				'excerpt',
				'author',
			),
			'show_ui' => true,
			'public' => true,
			'has_archive' => true,
			'show_in_nav_menus' => true,
            'rewrite' => array(
                'slug' => $archive_rewrite_slug,
            ),
			'taxonomies' => $taxonomies,
			'menu_icon' => 'dashicons-admin-home',
			'menu_position' => 32,
		);

		foreach($custom_post_types as $post_type_name => $args)
		{
			register_post_type($post_type_name, $args);
		}

	}

	public function register_custom_taxonomies() {

        /* Taxonomy - Agency */
		$labels = array(
			'name'                       => __( 'Agencies', YSD_MAPI_TEXT_DOMAIN ),
			'singular_name'              => __( 'Agency', YSD_MAPI_TEXT_DOMAIN ),
			'menu_name'                  => __( 'Agencies', YSD_MAPI_TEXT_DOMAIN ),
		);
		$args = array(
			'labels'                     => $labels,
			'hierarchical'               => true,
			'public'                     => true,
			'show_ui'                    => true,
			'show_admin_column'          => true,
			'show_in_nav_menus'          => true,
			'show_tagcloud'              => true,
            'query_var'                  => true,
		);
        register_taxonomy( 'agency', array( 'property' ), $args );

        /* Taxonomy - Offer */
		$labels = array(
			'name'                       => __( 'Offers', YSD_MAPI_TEXT_DOMAIN ),
			'singular_name'              => __( 'Offer', YSD_MAPI_TEXT_DOMAIN ),
			'menu_name'                  => __( 'Offers', YSD_MAPI_TEXT_DOMAIN ),
		);
		$args = array(
			'labels'                     => $labels,
			'hierarchical'               => true,
			'public'                     => false,
			'show_ui'                    => true,
			'show_admin_column'          => true,
			'show_in_nav_menus'          => true,
			'show_tagcloud'              => true,
            'query_var'                  => true,
            'rewrite' => array(
                'slug'                   => 'offer',
                'with_front'             => false,
                'feed'                   => false,
                'pages'                  => false
            )
		);
        register_taxonomy( 'offer', array( 'property' ), $args );

        /* Taxonomy - City */
		$labels = array(
			'name'                       => __( 'Cities', YSD_MAPI_TEXT_DOMAIN ),
			'singular_name'              => __( 'City', YSD_MAPI_TEXT_DOMAIN ),
			'menu_name'                  => __( 'Cities', YSD_MAPI_TEXT_DOMAIN ),
		);
		$args = array(
			'labels'                     => $labels,
			'hierarchical'               => true,
			'public'                     => false,
			'show_ui'                    => true,
			'show_admin_column'          => true,
			'show_in_nav_menus'          => true,
			'show_tagcloud'              => true,
            'query_var'                  => true,
            'rewrite' => array(
                'slug'                   => 'city',
                'with_front'             => false,
                'feed'                   => false,
                'pages'                  => false
            )
		);
        register_taxonomy( 'city', array( 'property' ), $args );

	}

	/**
     * Set Offer Tax_Query
     */
    function set_offer_tax_query( $query ) {

        // Check if Template is Buy
        $buy_template = get_option( 'ysd-mapi_buy_template' );

        if ( ! empty( $buy_template ) ) {

            // For Post Type 'Property' Only
            if ( ! is_admin() && 'property' == $query->get('post_type') ) {
                
                /* Offer: Koop */
                if ( is_page( $buy_template ) ) {
                    set_query_var( 'offer', __( 'buy', YSD_MAPI_TEXT_DOMAIN ) );

                    return $query;
                }
            }
        }

        // Check if Template is Rent
        $rent_template = get_option( 'ysd-mapi_rent_template' );

        if ( ! empty( $rent_template ) ) {

            // For Post Type 'Property' Only
            if ( ! is_admin() && 'property' == $query->get('post_type') ) {

                /* Offer: Huur */
                if ( is_page( $rent_template ) ) {
                    set_query_var( 'offer', __( 'rent', YSD_MAPI_TEXT_DOMAIN ) );

                    return $query;
                }
            }
        }

    }

	/**
     * Pretty Permalinks - Rewrite Rules Single
     */
    function set_property_single_slug( $post_link, $post ) {

        if ( 'property' != get_post_type( $post ) ) {
            return;
        }

        $post = get_post( $post );

        if ( is_object( $post ) ) {

            // Offer
            $terms = wp_get_object_terms( $post->ID, 'offer' );
            if ( $terms ) {
                $post_link = str_replace( '%offer%', $terms[0]->slug, $post_link );
            }

            // City
            $terms = wp_get_object_terms( $post->ID, 'city' );
            if ( $terms ) {
                $post_link = str_replace( '%city%', $terms[0]->slug, $post_link );
            }

            // Property Name
            $post_link = str_replace( '%propertyname%/', $post->post_name, $post_link );

        }

        return $post_link;  
    }

    /**
     * Pretty Permalinks - Rewrite Rules Archive
     */
	public function set_rewrite_rules() {

        $archive_rewrite_slug = get_option( 'ysd-mapi_archive_rewrite_slug', 'offer' );
        $single_rewrite_slug = get_option( 'ysd-mapi_single_rewrite_slug', 'property' );

        // URL Structure
        $single_url_structure = get_option( 'ysd-mapi_single_url_structure', 'plain' );

        // Add Permastructure
        if ( $single_url_structure != 'plain' ) {
            add_permastruct( 'property', $single_rewrite_slug .'/%offer%/%city%/%propertyname%/', false );
        }

        // Rewrite Tag - Offer
        add_rewrite_tag(
            '%offer%',
            '([^/]+)',
            'offer='
        );

        // Rewrite Tag - City
        add_rewrite_tag(
            '%city%',
            '([^/]+)',
            'city='
        );

        // Rule (single) - /Offer/City
        if ( $single_url_structure != 'plain' ) {
            add_rewrite_rule(
                '^'. $single_rewrite_slug .'/([^/]+)/([^/]+)/([^/]+)/?$',
                'index.php?post_type=property&offer=$matches[1]&city=$matches[2]&property=$matches[3]',
                'top'
            );
        }

        // Rule (archive) - /Offer/City
        add_rewrite_rule(
            '^'. $archive_rewrite_slug .'/([^/]+)/([^/]+)/page/?([0-9]{1,})/?$',
            'index.php?post_type=property&offer=$matches[1]&city=$matches[2]&paged=$matches[3]',
            'top'
        );
        add_rewrite_rule(
            '^'. $archive_rewrite_slug .'/([^/]+)/([^/]+)/?$',
            'index.php?post_type=property&offer=$matches[1]&city=$matches[2]',
            'top'
        );

        // Rule (archive) - /Offer
        add_rewrite_rule(
            '^'. $archive_rewrite_slug .'/([^/]+)/page/?([0-9]{1,})/?$',
            'index.php?post_type=property&offer=$matches[1]&paged=$matches[2]',
            'top'
        );
        add_rewrite_rule(
            '^'. $archive_rewrite_slug .'/([^/]+)/?$',
            'index.php?post_type=property&offer=$matches[1]',
            'top'
        );

    }
	
	/**
     * Pretty Permalinks - Query Vars
     */
	public function set_query_vars( $vars ) {

        $vars[] = 'offer';
        $vars[] = 'city';

        return $vars;
    }

	public function custom_excerpt_length($length)
	{
		global $post;

		if( !empty($post) && $post->post_type == 'property' )
		{
			$length = 70;
		}

		return $length;
	}

	public function custom_excerpt_more($more)
	{
		global $post;

		if( !empty($post) && $post->post_type == 'property' )
		{
			$more = ' <a class="property-more" href="' . get_the_permalink() . '">[&hellip;]</a>';
		}

		return $more;
	}
	#endregion

	#region media
	public function load_media_library()
	{
		add_action('pre_get_posts', [$this->admin, 'filter_media']);
	}

	/**
	 * Method so attachments can still be deleted when using WPML plugin
	 * 
	 * Hooked to delete_attachment
	 *
	 * @param mixed $id
	 * 
	 * @return void
	 */
	function deleteAttachmentTranslations($id)
	{
        remove_action('delete_attachment', [$this, 'deleteAttachmentTranslations']);

        $langs = apply_filters('wpml_active_languages', []);

        if (!empty($langs))
		{
            global $wp_filter;

            // We need to temporarly remove WPML's hook or they will cache the result and not delete the file
            // Sadly they used a filter on a non global class instance, so we cannot access it via regular wp hook
            $hooks = $wp_filter['wp_delete_file']->callbacks[10];
            unset($wp_filter['wp_delete_file']->callbacks[10]);

            foreach ($langs as $code => $info)
			{
                $post_id = apply_filters('wpml_object_id', $id, 'attachment', false, $code);

                if ($id == $post_id)
				{
                    continue;
                }

                wp_delete_attachment($post_id, true);
            }

            $wp_filter['wp_delete_file']->callbacks[10] = $hooks;
        }

        add_action('delete_attachment', [$this, 'deleteAttachmentTranslations']);

    }

	/**
	 * Returns only the mapi sizes when image is attached to property post type
	 * 
	 * Hooked to intermediate_image_sizes_advanced
	 *
	 * @param array $sizes
	 * @param array $image_meta
	 * @param int $attachment_id
	 * 
	 * @return array Filtered list of image sizes
	 */
	public function filterImageSizes($sizes, $image_meta, $attachment_id)
	{
		$attachment = get_post($attachment_id);

		if(isset($attachment->post_parent) && get_post_type($attachment->post_parent) == "property")
		{
			/* Return only the sizes that contain the word mapi (exception on thumbnail) */
			$sizes = array_filter($sizes, function($key) {
				return strpos($key, 'mapi') !== false || $key == "thumbnail";
			}, ARRAY_FILTER_USE_KEY); 
		}

		return $sizes;

	}
	#endregion
	
	/**
	 * immediate sync api data
	 *
	 * @param [type] $_GET
	 * 	 */
	public function immediate_sync_api_data() {

        $permission = check_ajax_referer( 'ajax_run_mapi_cron_nonce', 'nonce', false );
        
        if ( $permission ) {
            if ( class_exists( 'PropertyHandler' ) ) {
                $property_handler = new PropertyHandler();
                $property_handler->sync_api_data();
                echo 'Good';
            } else {
                echo 'Wrong';
            }
            echo 'success';
        } else {
            echo 'error';
        }
    
        die();

	}
	#endregion

	#region getters/setters
	/**
	 * The name of the plugin used to uniquely identify it within the context of
	 * WordPress and to define internationalization functionality.
	 *
	 * @since     1.0.0
	 * @return    string    The name of the plugin.
	 */
	public function get_plugin_name()
	{
		return $this->plugin_name;
	}

	/**
	 * The reference to the class that orchestrates the hooks with the plugin.
	 *
	 * @since     1.0.0
	 * @return    Loader    Orchestrates the hooks of the plugin.
	 */
	public function get_loader()
	{
		return $this->loader;
	}

	/**
	 * Retrieve the version number of the plugin.
	 *
	 * @since     1.0.0
	 * @return    string    The version number of the plugin.
	 */
	public function get_version()
	{
		return $this->version;
	}
	#endregion

}
