<?php
namespace ysdmapi\includes;

use WP_Error;

class MediaLoader
{
    /**
     * Tries to load the media from the given URL and imports it into wordpress
     * as an attachment
     *
     * Uses wordpress core method: media_handle_sideload to import the media
     * 
     * @param string $media_url
     * The URL to load the media from
     * @param string $media_source
     * The media source. Used for checking wheter the media should be loaded.
     * @param integer $post_id
     * If given attaches the media to the post
     * 
     * @return integer|WP_Error
     * The attachment (post) id or WP_Error on failure
     */
    public function sideload_url(string $media_url, string $media_source = "", string $media_alias, int $post_id = 0)
    {
        require_once(ABSPATH . 'wp-admin/includes/media.php');
		require_once(ABSPATH . 'wp-admin/includes/file.php');
        require_once(ABSPATH . 'wp-admin/includes/image.php');
        
        $media_name = pathinfo(strtok($media_url, '?'))['basename'];
        $media_alias = (!empty($media_alias)) ? $media_alias : $media_name;

        /* If Media Allowed */
        if(!$this->media_allowed($media_source)) {

            // WP_Error
            return new WP_Error(
                'media_import_skipped',
                sprintf(
                    __('Skipped due to import settings: %s', 'ysd-mapi'),
                    $media_alias
                ),
                [
                    'timestamp' => time(),
                    'severity_level' => 'info',
                    'media_name' => $media_alias,
                    'group_by' => 'media_name',
                    'property_id' => $post_id,
                ]
            );
        }
        
        /* If Media Exists */
        if(true == $this->media_exists($media_alias)) {

            // WP_Error
            return new WP_Error(
                'media_import_exists',
                sprintf(
                    __('Media already in library: %s', 'ysd-mapi'),
                    $media_alias
                ),
                [
                    'timestamp' => time(),
                    'severity_level' => 'info',
                    'media_name' => $media_alias,
                    'group_by' => 'media_name',
                    'property_id' => $post_id,
                ]
            );
        }

        $media_tmp = download_url($media_url);
        
        if(is_wp_error($media_tmp)) {
            $media_tmp->add_data([
                'timestamp' => time(),
                'severity_level' => 'error',
                'media_name' => $media_alias,
                'property_id' => $post_id,
            ]);

            ob_start();
            print_r($media_tmp->get_all_error_data());
            $output = ob_get_flush();
            error_log($output);
            
            return $media_tmp;
        }

        $media_size = filesize($media_tmp);

        $file = [
            'name' => $media_alias,
            'tmp_name' => $media_tmp,
            'error' => 0,
            'size' => $media_size,        
        ];

        /* Attach the media to the property */
        $attachment_id = media_handle_sideload($file, $post_id, null, [
            'post_name' => $media_alias,
            'post_title' => $media_alias,
            'post_excerpt' => $media_source,
            'post_content' => __('Imported by MAPI', 'ysd-mapi'),
        ]);

        if(is_wp_error($attachment_id)) {
            $attachment_id->add_data([
                'timestamp' => time(),
                'severity_level' => 'error',
                'media_name' => $media_alias,
                'property_id' => $post_id,
            ]);

            return $attachment_id;
        }

        /* Set the image as featured image if there isn't already a featured image */
        if($post_id != 0) {
            if(!has_post_thumbnail($post_id)) {
                set_post_thumbnail($post_id, $attachment_id);
            }
        }

        /* Attach show_in_library metadata to image */
        $show_in_library = get_option('ysd-mapi_show_in_library');
        update_post_meta( $attachment_id, 'show_in_library', $show_in_library);

        return $attachment_id;

    }

    /**
     * Check if option for media type is on or off
     *
     * @return boolean
     */
    private function media_allowed(string $media_source) : bool
    {
        $result = false;

        switch($media_source)
        {
            case "SOURCE_PROPERTY_IMAGE":
                $result = get_option('ysd-mapi_load_images');
                break;
            case "SOURCE_PROPERTY_VIDEO":
                $result = get_option('ysd-mapi_load_videos');
                break;
            case "SOURCE_PROPERTY_FLOORPLAN":
                $result = get_option('ysd-mapi_load_floorplans');
                break;
            case "SOURCE_PROPERTY_DOCUMENT":
                $result = get_option('ysd-mapi_load_brochures');
                break;
        }

        return $result;
        
    }

    private function media_exists(string $filename) : bool
    {
        $result = false;

        $file = explode('.', $filename);
        $name = $file[0];
        $extension = '.' . $file[1];

        if ( ! empty( $filename ) ) {
            global $wpdb; 
            $query = "SELECT post_id, meta_id FROM {$wpdb->postmeta} WHERE meta_value LIKE '%/$filename' OR meta_value LIKE '%/" . $name . '-scaled' . $extension . "'";

            if ( ! empty( $wpdb->get_var( $query ) ) ) {
                $result = true;
            }
        }

        return $result;

    }

}