<?php

namespace ysdmapi\includes;

use WP_Query;
use ysdmapi\includes\post_models\Property;
use ysdmapi\includes\MediaLoader;

class PropertyHandler
{
    public string $plugin_name;

	public JsonLog $status_log;

	public JsonLog $data_log;

	protected $process_all;


	public function __construct() {
		if(file_exists(YSD_MAPI_UPLOAD_DIR . '/logs/last/status.log'))
		{
			$this->status_log = unserialize(file_get_contents(YSD_MAPI_UPLOAD_DIR . '/logs/last/status.log'));
		}

		if(file_exists(YSD_MAPI_UPLOAD_DIR . '/logs/last/data.log'))
		{
			$this->data_log = unserialize(file_get_contents(YSD_MAPI_UPLOAD_DIR . '/logs/last/data.log'));
		}

		$this->process_all = new ImportProperty($this);


	}

    #region api syncing
	/**
	 * Fetches and processes the data from the api
	 *
	 * @return void
	 */
	public function sync_api_data() : void
	{
		$upload_dir_array = wp_get_upload_dir();
		$upload_dir = $upload_dir_array['basedir'] . '/' . $this->plugin_name;

		$this->create_log_files();

		$this->status_log->update_content(['status' => 0]);

		/* Which API Handler URL */
        $api_handler_url = get_option( 'ysd-mapi_api_handler_url', 0 );
        switch($api_handler_url) {
            case 0:
                $api_handler_url = 'https://portal.makelaarsapi.nl';
                break;
            case 1:
                $api_handler_url = 'https://portal.recreatievastgoed.nl';
                break;
            default:
                $api_handler_url = 'https://portal.makelaarsapi.nl';
        }

		/* Call the API */
		$api_handler = new ApiHandler($api_handler_url . '/api/agencies/' . get_option($this->plugin_name . '_mapi_token'));
		$data = $api_handler->fetch_data();

		/* If API call failed return WP_Error and update log */
		if(is_wp_error($data))
		{
			$this->status_log->update_content(['status' => -1]);

			$this->status_log->add_content([
				'messages' => [
					[
						'timestamp' => time(),
						'message' => $data->get_error_message(),
						'severity_level' => 'error',
					],
				],
			]);

			return;
		}

		$data = $data['data']['properties'];

		/* Use test data if it exists */
		if(file_exists($upload_dir . '/test-files/test_api_data.json'))
		{
            // @todo: Make an index check here
			$data = json_decode(file_get_contents($upload_dir . '/test-files/test_api_data.json'), true)['properties'];
		}

		$this->process_properties($data, false);

	}

	public function process_properties($data, $new_logs = true)
	{
		if($new_logs)
		{
			$this->create_log_files();
		}

		/* Write the retrieved data to the data log */
		$this->data_log->update_content($data);

		/* Process the data */
		if(!empty($data))
		{
			$this->status_log->update_content(['status' => 1]);
			$this->status_log->update_content(['total' => count($data)]);

			$this->process_next_property();
		}

	}

	private function create_log_files()
	{
		/* Get upload dir path */
		$upload_dir_array = wp_get_upload_dir();
		$upload_dir = $upload_dir_array['basedir'] . '/' . $this->plugin_name;

		/* Create status log file */
		$this->status_log = new JsonLog($upload_dir . '/logs', 'crm_sync_status');

		/* Write the status_log object to a file so it can be accessed from the import job */
		if(!file_exists($upload_dir . '/logs/last/status.log'))
		{
			$dirname = dirname($upload_dir . '/logs/last/status.log');

			/* Ensure the directory exists */
			if (!is_dir($dirname))
			{
				mkdir($dirname, 0755, true);
			}
		}

		/* Create the file */
		$file = fopen($upload_dir . '/logs/last/status.log', 'w');
		fwrite($file, serialize($this->status_log)); 
		fclose($file);

		/* Use status_log timestamp so the timestamps of both logs are always equal */
		$this->data_log = new JsonLog($upload_dir . '/logs', 'crm_data', true, $this->status_log->get_timestamp());
		
		/* Write the data_log object to a file so it can be accessed from the import job */
		if(!file_exists($upload_dir . '/logs/last/data.log'))
		{
			$dirname = dirname($upload_dir . '/logs/last/data.log');

			/* Ensure the directory exists */
			if (!is_dir($dirname))
			{
				mkdir($dirname, 0755, true);
			}
		}

		/* Create the file */
		$file = fopen($upload_dir . '/logs/last/data.log', 'w');
		fwrite($file, serialize($this->data_log)); 
		fclose($file);

	}

	public function process_next_property()
	{
		/* Properties from the data_log */
		$properties = $this->data_log->get_content();
		/* All info from the status log */
		$status = $this->status_log->get_content();

		if(!isset($status['index']))
		{
			$status['index'] = 0;
			$this->status_log->update_content(['index' => 0]);
		}

		/* Get the current index of the property being processed */
		$index = intval($status['index']);

		if($index < $status['total'])
		{
			$property_data = $properties[$index];

			/* Initialize var */
			$property_content = '';

			/* 
			*	Set the api description as property content and unset so 
			*	$property_data can be used as meta_data
			*/
			if(isset($property_data['description']))
			{
				$property_content = $property_data['description'];
				unset($property_data['description']);
			}

			/* Set sync_api meta data, so the added properties will be synced with the API */
			$property_data['sync_api'] = 'true';

			/* Write the API data to a property Model */
			$property = new Property(NULL, $property_content, $property_data);

			$property_old_meta = $property->get_db_post_meta();

			/* Save the property to the database */
			$result = $property->save();

			if(is_wp_error($result))
			{
				// Error
				$this->status_log->add_content([
					'messages' => [
						[
							'code' => 'property_import_error',
							'category' => 'property',
							'timestamp' => time(),
							'message' => sprintf(
								__(
                                    'Something went wrong trying to update property: %s',
                                    YSD_MAPI_TEXT_DOMAIN
								),
								$property->get_id()
							),
							'severity_level' => 'error',
						],
					],
				]);
			}
			else if(!empty($property_old_meta) && is_int($result))
			{
				// Updated
				$this->status_log->add_content([
					'messages' => [
						[
							'code' => 'property_import_updated',
							'category' => 'property',
							'timestamp' => time(),
							'message' => sprintf(
                                __('Updated property: %s',
                                YSD_MAPI_TEXT_DOMAIN
                            ),
                            $result
							),
							'severity_level' => 'info',
							'data' => [
								'property_id' => $result,
							],
						]
					],
				]);
			}
			else if(is_int($result))
			{
				// Added
				$this->status_log->add_content([
					'messages' => [
							[
							'code' => 'property_import_added',
							'category' => 'property',
							'timestamp' => time(),
							'message' => sprintf(
								__(
									'Added new property: %s',
									YSD_MAPI_TEXT_DOMAIN
								),
								$result
							),
							'severity_level' => 'info',
							'data' => [
								'property_id' => $result,
							],
						],
					],
				]);
			}

			/* Process the media inside a queue */
			if(isset($property_data['media']) && !empty($property_data['media']))
			{
				$this->status_log->update_content(['media_progress' => '0']);
				$this->status_log->update_content(['media_total' => count($property_data['media'])]);

				foreach($property_data['media'] as $media)
				{
					$media['property_id'] = $property->get_id();
					$this->process_all->push_to_queue( $media );
				}

				$this->process_all->save()->dispatch();
			}
			else
			{
				/* Increase the index for the next iteration */
				$this->status_log->update_content(['index' => $index + 1]);

				$this->process_next_property();
			}
		}
		else
		{
			$this->remove_deleted_properties($properties);
		}

	}

	/**
	 * Removes the properties from the database that aren't in the provided new_data array
	 *
	 * @param array $new_data
	 * An array of properties, containing the properties that shouldn't be deleted.
	 * 
	 * @param boolean $force_delete
	 * Whether to bypass Trash and force the deletion of the properties. Default false.
	 * 
	 * @return void
	 */
	public function remove_deleted_properties(array $new_data, bool $force_delete = false)
	{
		$this->status_log->update_content(['status' => 2]);

		/* Get the current stored properties */
		$query = new WP_Query(array (
			'post_type' => 'property',
			'posts_per_page' => -1,
			'meta_query' => array (
				array (
					/* Only the ones that are imported from the API */
					'key' => 'external_id',
					'compare' => 'EXISTS',
				),
				array (
					/* Only the ones that should be synced with the API */
					'key' => 'sync_api',
					'compare' => '=',
					'value' => 'true',
				),
			),
		));

		/* Loop through the properties */
		while($query->have_posts())
		{
			$query->the_post();

			$id = get_the_ID();
			$external_id = get_post_meta($id, 'external_id', true);
			$sync_values[] = get_post_meta($id, 'sync_api', true);

			$delete = true;

			foreach ($new_data as $new_meta_values)
			{
				if($external_id == $new_meta_values['id'])
				{
					$delete = false;

					break;
				}
			}

			if($delete)
			{
				wp_delete_post($id, $force_delete);
			}
		}

		$this->status_log->update_content(['status' => 3]);

    }
	#endregion

	#region media
	/**
	 * Deletes all media attached to the provided property
	 *
	 * Hooked to before_delete_property
	 * 
	 * @param int $property_id
	 * @return void
	 */
	public function delete_property_media(int $property_id) : void
	{
		if( get_post_type($property_id) == "property" ) 
		{
		  	$attachments = get_attached_media( '', $property_id );
	  
		  	foreach ($attachments as $attachment) {
				wp_delete_attachment( $attachment->ID, true );
			}
		}
	  
	}

	public function update_show_in_library() : void
	{
        $show_in_library = get_option('ysd-mapi_show_in_library');

		$query = new WP_Query(array (
			'post_type' => 'property',
			'posts_per_page' => -1,
			'meta_query' => array (
				array (
					/* Only the ones that are imported from the API */
					'key' => 'external_id',
					'compare' => 'EXISTS',
				),
			),
		));

		while($query->have_posts())
		{
			$query->the_post();

			$id = get_the_ID();

			$attachments = get_attached_media( '', $id );

			foreach ($attachments as $attachment)
			{
				update_post_meta($attachment->ID, 'show_in_library', $show_in_library);
			}
		}

	}
	#endregion

}