<?php

namespace ysdmapi\includes\custom_fields;

class CustomField
{
    /**
     * The unique field name. Make sure it corresponds with a potential associated meta key
     *
     * @var string
     */
    public $id;

    /**
     * The display name for the field
     *
     * @var string
     */
    public $label;

    /**
     * The type of the field. Can either correspond to a file in partials/custom-fields or
     * to a standard HTML input type
     *
     * @var string
     */
    public $type;

    /**
     * Method that checks if the input is valid before saving.
     * Should return true on valid and false on invalid.
     * 
     * @var callable
     */
    public $validation;

    /**
     * The value of the field, usually used inside the value attribute of the input
     *
     * @var string|null
     */
    public $value;

    /**
     * The possible options in case of a select field. Options can be grouped by
     * nesting them in arrays
     *
     * @var array
     */
    public $options;

    public $emptyLabelMin;
    
    public $emptyLabelMax;

    public $optionsMin;
    
    public $optionsMax;

    public $parent;

    public $pages;

    public $attributes;

    public $attributesString;

    public $condition;

    public $description;

    /**
     * @param string $id
     * The unique field name. Make sure it corresponds with a potential associated meta key.
     * Usually used in the name attribute and the id attribute of the input.
     * 
     * @param string $label
     * The display name for the field
     * 
     * @param string $type
     * The type of the field. Can either correspond to a file in partials/custom-fields or
     * to a standard HTML input type
     * 
     * @param callable $validation
     * Method that checks if the input is valid before saving.
     * Should return true on valid and false on invalid.
     * 
     * @param mixed $value
     * The value of the field, usually used inside the value attribute of the input
     * 
     * @param array $options
     * The possible options in case of a select field. Options can be grouped by
     * nesting them in arrays
     * 
     * @param array $attributes
     * Optional attributes that will be added to the input element of the field
     * 
     * @param array $condition
     * Optional condition that will determine wether the field is hidden or not
     * 
     */
    public function __construct($id, $label, $type, $validation = NULL, $value = NULL, $options = NULL, $emptyLabelMin = NULL, $emptyLabelMax = NULL, $optionsMin = NULL, $optionsMax = NULL, $description = null, $attributes = NULL, $condition = NULL)
    {
        $this->id = $id;
        $this->label = $label;
        $this->type = $type;
        $this->validation = $validation;
        $this->value = $value;
        $this->options = $options;
        $this->emptyLabelMin = $emptyLabelMin;
        $this->emptyLabelMax = $emptyLabelMax;
        $this->optionsMin = $optionsMin;
        $this->optionsMax = $optionsMax;
        $this->parent = NULL;
        $this->pages = array();
        $this->attributes = $attributes;
        $this->condition = $condition;
        $this->description = $description;

        if(isset($attributes) && is_array($attributes))
        {
            foreach($attributes as $atr_key => $atr_value) {
                $this->attributesString .= $atr_key . '="' . esc_attr($atr_value) . '" ';
            }
        }
    }

    /**
     * Handles the display of the HTML for the field
     *
     * @return void
     */
    public function display($post = null)
    {
        $data['post'] = $post;
        $data['field'] = $this;

        $path_to_html = YSD_MAPI_PLUGIN_PATH . 'includes/custom_fields/partials/fields/' . $this->type . '.php';

        if(!empty($this->condition))
        {
            $field = 'data-condition-field="' . $this->condition['field'] . '"' ?? '';
            $value = 'data-condition-value="' . $this->condition['value'] . '"' ?? '';
            $compare = $this->condition['compare'] = 'data-condition-compare="' . $this->condition['compare'] . '"' ?? '';

            $conditional = '<div class="mapi-filter__field" ' . $field  . ' ' . $value . ' ' . $compare . ' >';
        }

        if(!file_exists($path_to_html))
        {
            $path_to_html = YSD_MAPI_PLUGIN_PATH . 'includes/custom_fields/partials/fields/default.php';
        }
        
        echo $conditional ?? '<div class="mapi-filter__field">';
            include $path_to_html;
        echo '</div>';
    }

    /**
     * Returns the formatted value to be placed inside the value attribute of the input
     *
     * @return void
     */
    public function get_value($key = null)
    {
        $value = $this->value;

        if(isset($key) && isset($value[$key]))
        {
            $value = $value[$key];
        }

        switch($this->type)
        {
            case 'datetime-local':
                $value = strftime('%Y-%m-%dT%H:%M:%S', strtotime($value));
            break;
        }

        return $value;

    }

    public function sanitize_value($value)
    {
        sanitize_text_field($value);
        
        switch($this->type)
        {
            case 'datetime-local':
                $value = strftime('%Y-%m-%d %H:%M:%S', strtotime($value));
            break;
        }

        return $value;
        
    }

}