<?php
namespace ysdmapi\includes\custom_fields;

class CustomMetaBox
{
    public $id;

    public $title;

    public $screens;

    public $type;

    public $context;

    public $priority;

    protected $custom_field_groups;

    /**
     * @param string $id
     * Meta box ID (used in the 'id' attribute for the meta box).
     * 
     * @param string $title
     * Title of the meta box.
     * 
     * @param string|array $screens       
     * The screen or screens on which to show the box (such as a post type, 'link', or 'comment').
     * Accepts a single screen ID, WP_Screen object, or array of screen IDs.
     * If you have used add_menu_page() or add_submenu_page() to create a new screen (and hence screen_id),
     * make sure your menu slug conforms to the limits of sanitize_key() otherwise the 'screen' menu may not
     * correctly render on your page.
     * 
     * @param string $type
     * The meta box type that corresponds with a file in partials/meta_boxes
     * 
     * @param string $context       
     * Optional. The context within the screen where the boxes should display. Available contexts vary from
     * screen to screen. Post edit screen contexts include 'normal', 'side', and 'advanced'. Comments
     * screen contexts include 'normal' and 'side'. Menus meta boxes (accordion sections) all use the 'side'
     * context. Global default is 'advanced'.
     * 
     * @param string $priority
     * Optional. The priority within the context where the boxes should show ('high', 'low'). Default 'default'.
     * 
     */
    public function __construct($id, $title, $screens, $type, $context = "advanced", $priority = "default")
    {
        $this->id = $id;
        $this->title = $title;
        $this->screens = $screens;
        $this->type = $type;
        $this->context = $context;
        $this->priority = $priority;
        $this->custom_field_groups = array();
    }

    /**
     * Add an array of CustomFieldGroup objects to the meta box
     *
     * @param CustomFieldGroup[] $field_groups
     * 
     * @return void
     */
    public function add_custom_field_groups(array $field_groups) : void
    {
        foreach($field_groups as $field_group)
        {
            $this->add_custom_field_group($field_group);
        }
    }

    /**
     * Add a CustomFieldGroup object to the meta box
     *
     * @param CustomFieldGroup $field_group
     * 
     * @return void
     */
    public function add_custom_field_group(CustomFieldGroup $field_group) : void
    {
        $field_group->set_parent($this);
        $field_group->add_pages($this->screens);
        $this->custom_field_groups[] = $field_group;
    }

    /**
     * Handles the saving of all the fields from the meta box
     * 
     * Saves the fields as post meta using the values inside $_POST.
     * 
     * @param int $post_id
     * The post id to save the fields to as post_meta
     *
     * @return void
     */
    public function save_fields($post_id)
    {
        $screens_array = is_array($this->screens) ? $this->screens : array($this->screens);
        $current_screen = '';

        if(function_exists('get_current_screen'))
        {
            $current_screen = get_current_screen()->id ?? '';
        }

        /**
         * Only save when it's done from one of the specified screens
         */
        if(in_array($current_screen, $screens_array))
        {
            $fields = $this->get_fields();
            
            foreach($fields as $field)
            {
                if (array_key_exists($field->id, $_POST)) {
                    if(isset($field->validation) && !FieldValidator::validate_field($field, $_POST[$field->id]))
                    {
                        //@todo return wp error message
                        return false;
                    }

                    /* Sanitize before saving */
                    $value = $field->sanitize_value($_POST[$field->id]);

                    update_post_meta( $post_id, $field->id, $value );
                }
                else
                {
                    /**
                     * Remove from meta when the field is not provided
                     */
                    delete_post_meta( $post_id, $field->id);
                }
            }
        }
        
    }

    /**
     * Returns all the meta box CustomFieldGroup objects as an array.
     * 
     * Returns empty array if none exist
     *
     * @return CustomFieldGroup[]
     */
    public function get_field_groups() : array
    {
        return $this->custom_field_groups;
    }

    /**
     * Returns all the meta box CustomField objects as an array.
     * 
     * Returns empty array if none exist
     *
     * @return CustomField[]
     */
    public function get_fields() : array
    {
        $fields = array();

        foreach($this->custom_field_groups as $field_group)
        {
            foreach($field_group->get_fields() as $field)
            {
                $fields[] = $field;
            }
        }

        return $fields;
        
    }

    /**
     * Register the custom meta box with WordPress
     *
     * @return void
     */
    public function register()
    {
        $data = array(
            'custom_field_groups' => $this->custom_field_groups,
            'type' => $this->type,
        );

        $title = __($this->title, YSD_MAPI_TEXT_DOMAIN);
        add_meta_box($this->id, $title, array($this, 'display'), $this->screens, $this->context, $this->priority, $data);
        
    }

    /**
     * Callback for the add_meta_box method
     * 
     * Displays the HTML for the meta box
     *
     * @param [type] $post
     * @param [type] $box
     * @return void
     */
    public function display($post, $box)
    {
        $fields = $this->get_fields();
        
        /*
         * Set the value of the fields equal to the post meta that corresponds to the field id
         * if there is any
         */
        foreach($fields as $field)
        {
            $meta = get_post_meta($post->ID, $field->id, true);
            
            if($meta !== false)
            {
                $field->value = $meta;
            }
        }

        /* the data to be used inside the html */
        $data = $box['args'];
        $data['post'] = $post;

        require YSD_MAPI_PLUGIN_PATH . 'includes/custom_fields/partials/meta_boxes/' . $data['type'] . '.php';

    }

}